package edu.ksu.cis.ndpc.gui;

import java.util.ArrayList;
import java.util.Arrays;

public class GUIBox {
	public static final int SHIFT_UP = -1;
	public static final int SHIFT_DOWN = 1;

	private ArrayList children;
	private int startRow;
	private int endRow;
	private Boundary bounds;
	private int height;

	public GUIBox(int startRow, int endRow) {
		this(startRow, endRow, new Boundary());
	}

	public GUIBox(int startRow, int endRow, Boundary bounds) {
		this.startRow = startRow;
		this.endRow = endRow;
		this.children = new ArrayList(0);
		this.bounds = bounds;
		this.height = 0;
	}

	public int startRow() {
		return startRow;
	}

	public int endRow() {
		return endRow;
	}

	public int height() {
		return height;
	}

	public boolean overlaps(GUIBox box) {
		if (box == null) {
			return false;
		} else {
			return (startRow < box.startRow && endRow > box.startRow && endRow < box.endRow)
					|| (startRow > box.startRow && startRow < box.endRow && endRow > box.endRow);
		}
	}

	public boolean strictOverlaps(GUIBox box) {
		return overlaps(box) || equals(box);
	}

	public boolean equals(Object b) {
		boolean equal = false;
		try {
			GUIBox box = (GUIBox) b;
			equal = (startRow == box.startRow) && (endRow == box.endRow);
		} catch (Exception e) { /* Do nothing since equal is still false */
		}

		return equal;
	}

	public boolean contains(int row) {
		return (row >= startRow && row < endRow);
	}

	public boolean contains(int startRow, int endRow) {
		boolean contains = false;
		for (int i = 0; !contains && i < children.size(); ++i) {
			GUIBox b = (GUIBox) children.get(i);
			contains = (b.startRow == startRow && b.endRow == endRow)
					|| b.contains(startRow, endRow);
		}
		return contains;
	}

	public boolean contains(GUIBox box) {
		if (box == null) {
			return false;
		} else {
			return contains(box.startRow, box.endRow);
		}
	}

	public void add(int startRow, int endRow) {
		this.add(new GUIBox(startRow, endRow));
	}

	public void add(GUIBox box) {
		// assert (this box does not contain 'box')
		boolean added = false;
		for (int i = 0; i < children.size(); ++i) {
			GUIBox b = (GUIBox) children.get(i);
			if (b.canContain(box)) { // box can be a child of b
				b.add(box);
				added = true;
				i = children.size();
			} else if (box.canContain(b)) { // b can be a child a box
				int pos = i;
				box.add(b);
				children.remove(i);
				while (i < children.size()) {
					b = (GUIBox) children.get(i);
					if (box.canContain(b)) {
						box.add(b);
						children.remove(i);
					} else {
						++i;
					}
				}
				children.add(pos, box);
				added = true;
			} else if (box.isBefore(b)) {
				children.add(i, box);
				added = true;
				i = children.size();
			}
		}
		if (!added) {
			children.add(box);
		}
		height = computeHeight();
	}

	public boolean remove(GUIBox box) {
		// assert (this does not contain box)
		boolean removed = false;
		int size = children.size();
		for (int i = 0; !removed && i < size; ++i) {
			GUIBox b = (GUIBox) children.get(i);
			if (b.equals(box)) {
				children.remove(i);
				if (b.containsBoxes()) {
					Object[] boxes = b.getChildren();
					for (int j = 0; j < boxes.length; ++j) {
						children.add(i++, boxes[j]);
					}
				}
				removed = true;
			} else if (b.canContain(box)) {
				removed = b.remove(box);
				i = size; // Exit out of the loop
			}
		}
		if (removed) { // If box was found and removed
			height = computeHeight();
		}
		return removed;
	}

	public boolean canContain(GUIBox box) {
		return (startRow <= box.startRow && endRow >= box.endRow);
	}

	public boolean containsBoxes() {
		return (children.size() > 0);
	}

	public boolean hasChildren() {
		return children.size() > 0;
	}

	public Object[] getChildren() {
		int size = children.size();
		Object[] boxes = new Object[size];
		for (int i = 0; i < size; ++i) {
			boxes[i] = children.get(i);
		}
		return boxes;
	}

	public boolean isBefore(GUIBox box) {
		return (endRow < box.startRow);
	}

	public void print(int spaces) {
		char[] c = new char[spaces];
		Arrays.fill(c, ' ');
		// System.out.print(c);
		// System.out.println("Begin Print boxes");
		// System.out.print(c);
		// System.out.print(" ");
		// System.out.println("UIBox["+startRow+","+endRow+"], height="+height);
		for (int i = 0; i < children.size(); ++i) {
			GUIBox box = (GUIBox) children.get(i);
			box.print(spaces + 2);
		}
		// System.out.print(c);
		// System.out.println("End Print boxes");
	}

	private int computeHeight() {
		int h = -1;
		for (int i = 0; i < children.size(); ++i) {
			GUIBox b = (GUIBox) children.get(i);
			int height = b.computeHeight();
			if (height > h) {
				h = height;
			}
		}
		return h + 1;
	}

	public Boundary getBounds() {
		return bounds;
	}

	public void setBounds(int x1, int y1, int x2, int y2) {
		bounds.set(x1, y1, x2, y2);
	}

	public void setBounds(Boundary bounds) {
		this.bounds = bounds;
	}

	public void shift(int rows, int direction) {
		startRow += (rows * direction);
		endRow += (rows * direction);
	}

	public void shiftBase(int rows, int direction) {
		// System.out.println(this + " shiftBase(" + rows + "," + direction +
		// ") begin: end="+endRow);
		endRow += (rows * direction);
		// System.out.println(this + " shiftBase(" + rows + "," + direction +
		// ") end: end="+endRow);
	}

	public void shiftTop(int rows, int direction) {
		// System.out.println(this + " shiftBase(" + rows + "," + direction +
		// ") begin: end="+endRow);
		startRow += (rows * direction);
		// System.out.println(this + " shiftBase(" + rows + "," + direction +
		// ") end: end="+endRow);
	}

	public void removeRow(int row) {
		// System.out.println(this + " removeRow " + row + " begin");
		if (containsRow(row)) {
			// System.out.println(" contains row");
			shiftBase(1, SHIFT_UP);
			for (int i = 0; i < children.size(); ++i) {
				GUIBox b = (GUIBox) children.get(i);
				b.removeRow(row);
				// if (b.endRow < b.startRow) {
				if (b.endRow == b.startRow) {
					children.remove(i);
					--i;
				} else if (b.equals(this)) {
					children = b.children;
					i = children.size();
				}
			}
			height = computeHeight();
		} else {
			// System.out.println(" does not contain row");
			if (startRow > row) {
				shift(1, SHIFT_UP);
			}
			for (int i = 0; i < children.size(); ++i) {
				GUIBox b = (GUIBox) children.get(i);
				b.removeRow(row);
			}
		}
		// System.out.println(this + " removeRow " + row + " end");
	}

	public void insertRow(int row) {
		// System.out.println("*InsertRow:"+this+", new row="+row);
		if (containsRow(row) && startRow <= row) {
			shiftBase(1, SHIFT_DOWN);
		} else if (startRow > row) {
			shift(1, SHIFT_DOWN);
		}
		for (int i = 0; i < children.size(); ++i) {
			GUIBox b = (GUIBox) children.get(i);
			b.insertRow(row);
		}
	}

	public boolean containsRow(int row) {
		return row >= startRow && row < endRow;
	}

	public String toString() {
		return "GUIBox[" + startRow + "," + endRow + "], height=" + height;
	}
}