/*
 * RawProof.java       Jan 19, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc.gui;

import java.io.Serializable;
import java.util.ArrayList;
import edu.ksu.cis.ndpc.Utils;

/**
 * Represents a proof as it appears on the graphical user interface.
 * 
 * A RawProof could contains zero or more statements and zero or more boxes. A
 * null statement indicates a blank line in the proof.
 * 
 * This proof is serialized to a file when a proof is saved to a file in
 * proprietary format.
 * 
 * 
 * @author Brian Mulanda (<a
 *         href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see RawStatement
 * @see RawBox
 * @see java.io.Serializable
 * 
 */

public final class RawProof implements Serializable {
	/**
	 * The class fingerprint that is set to indicate serialization compatibility
	 * with a previous version of the class.
	 */
	static final long serialVersionUID = 1;

	public transient static final int PROOF_UNKNOWN = -1;
	public transient static final int PROOF_PRED = 0;
	public transient static final int PROOF_PROP = 1;

	private transient static final String BLANK = "_";
	private transient static final String BOX_DELIM = ";";
	private transient static final String ITEM_DELIM = ":";
	private transient static final String PROOF_STRING_PRED = "Predicate Logic proof";
	private transient static final String PROOF_STRING_PROP = "Propositional Logic proof";

	private ArrayList statList;
	private ArrayList boxList;
	private int type;

	public RawProof(int type) {
		this.statList = new ArrayList(5);
		this.boxList = new ArrayList(5);
		this.type = type;
	}

	public void addStatement(RawStatement statement) {
		statList.add(statement);
	}

	public void addBox(RawBox box) {
		if (box != null)
			boxList.add(box);
	}

	public int getType() {
		return type;
	}

	public int[] boxesAtStat(int stat) {
		int[] box = { 0, 0 };
		if (boxList != null && stat >= 0 && stat < statList.size()) {
			for (int i = 0; i < boxList.size(); ++i) {
				RawBox rb = (RawBox) boxList.get(i);
				if (rb.startRow() == stat)
					(box[0])++;
				if (rb.endRow() == stat)
					(box[1])++;
			}
		}
		return box;
	}

	public RawStatement getStatement(int index) {
		if (index >= 0 && index < statList.size()) {
			return (RawStatement) statList.get(index);
		} else {
			return null;
		}
	}

	public RawBox getBox(int index) {
		if (index >= 0 && index < boxList.size()) {
			return (RawBox) boxList.get(index);
		} else {
			return null;
		}
	}

	public int size() {
		return statList.size();
	}

	public int boxCount() {
		return boxList.size();
	}

	public String toString() {
		StringBuffer proof = new StringBuffer();
		RawStatement statement = null;
		proof.append("Begin Proof").append(ITEM_DELIM).append(
				getTypeString(type)).append(ITEM_DELIM);

		if (boxList.size() > 0) {
			for (int i = 0; i < boxList.size(); ++i) {
				proof.append(boxList.get(i).toString()).append(BOX_DELIM);
			}
		} else {
			proof.append(BLANK);
		}
		proof.append(Utils.NEW_LINE);

		for (int i = 0; i < statList.size(); ++i) {
			statement = (RawStatement) statList.get(i);
			if (statement == null) {
				proof.append(BLANK);
			} else {
				proof.append(statement.toString());
			}
			proof.append(Utils.NEW_LINE);
		}
		proof.append("End Proof").append(Utils.NEW_LINE);
		return proof.toString();
	}

	public static RawProof fromString(String s) {
		RawProof proof = null;
		try {
			if (s != null) {
				String[] lines = s.split(Utils.NEW_LINE);
				if (lines.length > 0) {
					String[] strArr = lines[0].split(ITEM_DELIM);
					if (strArr.length > 0) {
						int type = getTypeValue(strArr[1]);
						if (type != RawProof.PROOF_UNKNOWN) {
							proof = new RawProof(type);

							if (strArr.length > 1 && !strArr[2].equals(BLANK)) {
								String[] boxes = strArr[2].split(BOX_DELIM);
								RawBox box = null;
								for (int i = 0; i < boxes.length; ++i) {
									box = RawBox.fromString(boxes[i]);
									if (box != null) {
										proof.addBox(box);
									}
								}
							}

							for (int i = 1; i < lines.length - 1; ++i) {
								proof.addStatement(RawStatement
										.fromString(lines[i]));
							}
						}
					}
				}
			}
		} catch (Exception e) {
			proof = null;
		}
		return proof;
	}

	public static String getTypeString(int type) {
		if (type == PROOF_PRED) {
			return PROOF_STRING_PRED;
		} else if (type == PROOF_PROP) {
			return PROOF_STRING_PROP;
		} else {
			return "Unknown";
		}
	}

	private static int getTypeValue(String type) {
		if (type.trim().equalsIgnoreCase(PROOF_STRING_PRED)
				|| type.trim().equalsIgnoreCase("Predicate")) {
			return PROOF_PRED;
		} else if (type.trim().equalsIgnoreCase(PROOF_STRING_PROP)
				|| type.trim().equalsIgnoreCase("Propositional")) {
			return PROOF_PROP;
		} else {
			return PROOF_UNKNOWN;
		}
	}
}
