package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofChecker;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleForallElim extends Rule {
	private static String name = getDescription(Rule.FORALL_ELIM);

	public RuleForallElim() {
		super();
	}

	public int getType() {
		return Rule.FORALL_ELIM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires a formula");
		} else if (!((params[0] instanceof ProofStatement) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires a formula");
		}

		ProofStatement premise = (ProofStatement) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = premise.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_FORALL) {
			throw new RuleException("Formula " + premise
					+ " must be universally quantified "
					+ "with the universal quantifier symbol and a variable. "
					+ "For example Ax(formula)");
		}

		ParseTree premiseTree = premise.getFormula().getParseTree();
		ParseTree conclusionTree = conclusion.getFormula().getParseTree();
		ParseTree premSubtree = premiseTree.getSubtree();

		if (!premSubtree.equals(conclusionTree)) {
			int freeCount = 0;
			Symbol var = new Symbol(premiseTree.getRootNode().getValue()
					.substring(1), LangSymbols.S_VAR);
			ParseTree termX = new ParseTree(var);

			String quantifier = premise.getFormula().getFormulaString()
					.substring(0, premiseTree.getRootNode().getLength()).trim();

			String formula = premise.getFormula().getFormulaString().substring(
					premiseTree.getRootNode().getLength()).trim();

			if (!ProofChecker.equalSyntax(premSubtree, conclusionTree)) {
				throw new RuleException("Incorrect substitution. "
						+ "The correct formula is obtained from " + premise
						+ " by first " + "removing '" + quantifier
						+ "', and then replacing all free occurrences "
						+ "of '" + termX + "' with a term that is free for '"
						+ termX + "' " + "in '" + formula + "'.");
			}

			freeCount = ProofChecker.countFreeIn(termX, premSubtree);
			if (freeCount == 0) {
				throw new RuleException("Incorrect substitution. "
						+ "There are no free occurrences of the term '" + termX
						+ "' in '" + formula + "'.\n"
						+ "The correct formula is obtained from " + premise
						+ " by first " + "removing '" + quantifier
						+ "', and then replacing all free occurrences "
						+ "of '" + termX + "' with a term that is free for '"
						+ termX + "' " + "in '" + formula + "'.");
			}

			ParseTree termT = ProofChecker.findTermT(premiseTree.getSubtree(),
					conclusionTree, termX);

			boolean ok = false;
			if (termT != null) {
				freeCount = freeCount
						- ProofChecker.countFreeForIn(termT, termX, premiseTree
								.getSubtree());
				if (freeCount != 0) {
					throw new RuleException("Incorrect substitution. "
							+ "The term '" + termT.allToString()
							+ "' is not free for '" + termX + "' in '"
							+ formula + "'.\n"
							+ "The correct formula is obtained from " + premise
							+ " by first " + "removing '" + quantifier
							+ "', and then replacing all free occurrences "
							+ "of '" + termX
							+ "' with a term that is free for '" + termX + "' "
							+ "in '" + formula + "'.");
				}

				freeCount = ProofChecker.countFreeForIn(termT, termX,
						conclusionTree);
				if (freeCount > 0) {
					throw new RuleException("Incorrect substitution. "
							+ "There is a free occurrence of '" + termX
							+ "' that has not been replaced by '"
							+ termT.allToString() + "'.\n"
							+ "The correct formula is obtained from " + premise
							+ " by first " + "removing '" + quantifier
							+ "', and then replacing all free occurrences "
							+ "of '" + termX + "' with '" + termT.allToString()
							+ "'.");
				}

				ParseTree testConclTree = ProofChecker.substitute(
						(ParseTree) premiseTree.getSubtree().clone(), termX,
						termT);

				ok = testConclTree.equals(conclusionTree);
			}
			if (!ok) {
				throw new RuleException("Incorrect substitution. "
						+ "The correct formula is obtained from " + premise
						+ " by first " + "removing '" + quantifier
						+ "', and then replacing all free occurrences "
						+ "of '" + termX + "' with a term that is free for '"
						+ termX + "' " + "in '" + formula + "'.");
			}
		}
	}
}